@@ -4,6 +4,9 @@ import os
 S3_ENCRYPT_KEY = os.environ.get("S3_ENCRYPT_KEY", '')
 DEV_SUFFIX = 'pre'
 
+RUN_TASK_LAMBDA_NAME = 'run_task'
+CHECK_TASK_LAMBDA_NAME = 'check_task'
+
 
 def BUCKET_NAME(env, filetype):
     if filetype in ['FileFastq', 'FileReference']:
